@@ -4,7 +4,7 @@ Business::Shipping::USPS_Online::Shipment
 
 =head1 VERSION
 
-$Rev: 194 $
+$Rev: 224 $
 
 =head1 DESCRIPTION
 
@@ -30,7 +30,7 @@ See Business::Shipping POD for usage information.
 
 package Business::Shipping::USPS_Online::Shipment;
 
-$VERSION = do { my $r = q$Rev: 194 $; $r =~ /\d+/; $&; };
+$VERSION = do { my $r = q$Rev: 224 $; $r =~ /\d+/; $&; };
 
 use strict;
 use warnings;
@@ -42,14 +42,16 @@ use Business::Shipping::USPS_Online::Package;
 use Class::MethodMaker 2.0 
     [ 
       new   =>  [ { -hash    => 1, -init => '_this_init' }, 'new' ],
+      new   =>  [ { -init    => '_this_init', }, 'default_new' ],
       array =>  [ { -type    => 'Business::Shipping::USPS_Online::Package',
-                    -default_ctor => 'new' }, 'packages' ],
+                    -default_ctor => 'default_new' }, 'packages' ],
       scalar => [ 'service' ],
       scalar => [ { -static  => 1, 
                     -default => 'packages=>Business::Shipping::USPS_Online::Package' 
                   }, 
                   'Has_a' 
                 ],
+      scalar => [ { -default => 70 }, 'max_weight' ],
 ];
 
 sub _this_init
